#! /bin/bash

set -eu

usage() {
    cat <<EOF
Usage: build.sh [--beta] [--debug] [--version VERSION]

--beta 	    Beta build (can also set CI_COMMIT_BRANCH to beta)
--debug	    Debug build (can also set DEBUG_BUILD)
--version   Version to build (can also set BUILD_VERSION)
EOF
    exit 1
}

fatal() {
    echo "$*" >&2
    exit 1
}

parse_args() {
    beta=""
    debug="${DEBUG_BUILD:-}"
    version="${BUILD_VERSION:-}"

    branch="${CI_COMMIT_BRANCH:-}"
    if [ "$branch" = "beta" ] ; then
	beta=yes
    fi

    while [ $# -gt 0 ] ; do
	case "$1" in
	    --version)
		version="$2"
		shift
		;;
	    --beta)
		beta=yes
		;;
	    --debug)
		debug=yes
		;;
	    --)
		shift
		break
		;;
	    -h|--help)
		usage
		;;
	    -*)
		usage
		;;
	    *)
		break
		;;
	esac
	shift
    done

    [ $# -eq 0 ] || usage
    [ "$version" ] || usage
}

setup_globals() {
    TARGETS=(thunderbird firefox edge)
    BASEDIR="$( cd "$(dirname "${BASH_SOURCE[0]}")/.." >/dev/null 2>&1 && pwd )"

    BUILDDIR="$BASEDIR/build"
    SHAREDDIR="$BASEDIR/shared"

    if [ -x "$BASEDIR/node_modules/.bin/web-ext" ] ; then
	WEB_EXT="$BASEDIR/node_modules/.bin/web-ext"
    else
	WEB_EXT="web-ext"
    fi

    if [ "$beta" ] ; then
	version_name="$version beta"
    else
	version_name="$version"
    fi
}

copy_source() {
    cp -r "$SHAREDDIR"/* "$targetdir"
    cp -r "$sourcedir"/* "$targetdir"
}

clean_debug() {
    if [ -z "${debug:-}" ] ; then
	for file in "$targetdir"/*.js ; do
	    sed -i -e '/\/\/ DEBUG$/d' "$file"
	done
    fi
}    

clean_target() {
    rm -f "$targetdir"/*.merge.json \
       "$targetdir"/*.override.json \
       "$targetdir"/BUILD \
       "$targetdir"/*~ \
       "$targetdir"/#*
}

build_manifests() {
    local merge=()
    local override=()
    local collect_merge
    local collect_override

    while [ $# -gt 0 ] ; do
	case "$1" in
	    --merge)
		collect="merge"
		;;
	    --override)
		collect="override"
		;;
	    *)
		[ "$collect" = "merge" ] && merge+=("$1")
		[ "$collect" = "override" ] && override+=("$1")
		;;
	esac
	shift
    done
    [ $# -eq 0 ] || fatal "internal error in call to build"

    python3 "$BASEDIR/scripts/makemanifest.py" \
	    --merge "${merge[@]}" --override "${override[@]}" \
	    > "$targetdir/manifest.json"

    sed -i -e "s|%XPIFILE%|$xpifile|g" "$targetdir/manifest.json"
    sed -i -e "s|%VERSION%|$version|g" "$targetdir/manifest.json"
    sed -i -e "s|%VERSION_NAME%|$version_name|g" "$targetdir/manifest.json"
    sed -i -e "s|%PAGES_URL%|$CI_PAGES_URL|g" "$targetdir/manifest.json"
    sed -i -e "s|%PROJECT_URL%|$CI_PROJECT_URL|g" "$targetdir/manifest.json"
}

build() {
    local merge=()
    local tmp_merge=()
    local override=()
    local tmp_override=()
    local quiet=

    while [ $# -gt 0 ] ; do
	case "$1" in
	    --target)
		target="$2"
		targetdir="$BUILDDIR/$2"
		[ "$sourcedir" ] || sourcedir="$targetdir"
		shift
		;;
	    --source)
		source="$2"
		sourcedir="$BASEDIR/$2"
		shift
		;;
	    --ext)
		ext="$2"
		shift
		;;
	    --merge)
		tmp_merge+=("$2")
		shift
		;;
	    --override)
		tmp_override+=("$2")
		shift
		;;
	    --quiet)
		quiet=yes
		;;
	    *)
		fatal "internal error in call to build: $*"
		;;
	esac
	shift
    done
    [ $# -eq 0 ] || fatal "internal error in call to build"

    local defaultmerge
    declare -a defaultmerge

    merge+=("$SHAREDDIR/manifest.merge.json")
    [ -f "$sourcedir/manifest.merge.json" ] && \
	merge+=("$sourcedir/manifest.merge.json")
    [ "$beta" -a -f "$sourcedir/manifest.beta.merge.json" ] && \
	merge+=("$sourcedir/manifest.beta.merge.json")
    [ "$beta" -a -f "$sourcedir/manifest.beta.override.json" ] && \
	merge+=("$sourcedir/manifest.beta.override.json")

    merge+=("${tmp_merge[@]}")
    override+=("${tmp_override[@]}")

    xpifile="safelinks-cleaner-$target.$ext"
    outputfile="$BUILDDIR/$xpifile"
    
    [ "$quiet" ] || echo "[+] building target '$target'"

    [ -d "$targetdir" ] && rm -r "$targetdir"
    mkdir -p "$targetdir"

    copy_source
    clean_target
    clean_debug
    build_manifests --merge "${merge[@]}" --override "${override[@]}"
}


parse_args "$@"
setup_globals

[ "$debug" ] && echo "[+] debugging statements enabled"

for target in "${TARGETS[@]}" ; do
    if [ -f "$target/BUILD" ] ; then
	. "$target/BUILD"
    else
	build --source "$target" --target "$target" --ext "xpi"
    fi
done
